
//
// MainForm.cs
//
// Author:
//    Tomas Restrepo (tomas.restrepo@devdeo.com)
//
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Text;
using System.Windows.Forms;
using Devdeo.StockQuotes.Client.Service;

namespace Devdeo.StockQuotes.Client
{
   /// <summary>
   /// Sample application Form. This form both displays
   /// the parameters needed to subscribe to the WCF service
   /// and receives the service notifications via the 
   /// IStockQuoteUpdateServiceCallback implementation
   /// </summary>
   public partial class MainForm : Form, IStockQuoteUpdateServiceCallback
   {
      private IStockQuoteUpdateService _service;
      private StockQuoteSubscription _subscription;

      public MainForm()
      {
         InitializeComponent();
         clientIdTextBox.Text = Guid.NewGuid().ToString();
      }

      private void OnStart(object sender, EventArgs e)
      {
         InstanceContext ctxt = new InstanceContext(this);
         _service = 
            new StockQuoteUpdateServiceClient(ctxt, "httpBasedBinding");

         _subscription = new StockQuoteSubscription();
         _subscription.ClientId = clientIdTextBox.Text;
         _subscription.Symbol = (string)symbolList.SelectedItem;
         _service.Subscribe(_subscription);
         
         startButton.Enabled = false;
         stopButton.Enabled = true;
      }

      private void OnStop(object sender, EventArgs e)
      {
         if ( _service != null )
         {
            _service.Unsubscribe(_subscription);
            ((IDisposable)_service).Dispose();
            _service = null;
            _subscription = null;
         }
         startButton.Enabled = true;
         stopButton.Enabled = false;
      }

      #region IStockQuoteUpdateServiceCallback Implementation
      //
      // IStockQuoteUpdateServiceCallback Implementation
      //
      delegate void QuoteChanged(StockQuoteUpdate update);

      public void OnQuoteChanged(StockQuoteUpdate update)
      {
         QuoteChanged del = ProcessQuote;
         Invoke(del, update);
      }

      private void ProcessQuote(StockQuoteUpdate update)
      {
         string[] textItems = new string[] 
         {
            update.LastUpdate.ToString(),
            update.Quote.ToString("C")
         };

         ListViewItem item = new ListViewItem(textItems);
         quoteList.Items.Add(item);
      }

      #endregion

   } // class MainForm

} // namespace Devdeo.StockQuotes.Client