
//
// ServiceImpl.cs
//
// Author:
//    Tomas Restrepo (tomas.restrepo@devdeo.com)
//
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using System.Xml;

namespace Devdeo.StockQuotes.Service
{
   [ServiceBehavior(InstanceContextMode=InstanceContextMode.PerSession, 
      AutomaticSessionShutdown=true)]
   public class StockQuoteUpdateService : IStockQuoteUpdateService, IDisposable
   {
      private IStockQuoteUpdateSink _client;
      private StockQuoteSubscription _subscription;

      #region IStockQuoteUpdateService Members
      //
      // IStockQuoteUpdateService Members
      //

      public void Subscribe(StockQuoteSubscription subscription)
      {
         Console.WriteLine("Client Connected");
         Console.WriteLine("  Symbol: {0}", subscription.Symbol);
         Console.WriteLine("  ClientId: {0}", subscription.ClientId);

         //
         // get client callback
         //
         OperationContext ctxt = OperationContext.Current;
         _client = ctxt.GetCallbackChannel<IStockQuoteUpdateSink>();
         Console.WriteLine("  Callback: {0}",
            ((IClientChannel)_client).RemoteAddress);

         _subscription = subscription;
         StockUpdateWatcher.Update += OnStockQuoteChange;
      }

      public void Unsubscribe(StockQuoteSubscription subscription)
      {
         _client = null;
         Console.WriteLine("Client Disconnected {0}", subscription.ClientId);
      }

      public void Dispose()
      {
         Console.WriteLine("Closing service");
         StockUpdateWatcher.Update -= OnStockQuoteChange;
      }

      #endregion

      #region Stock Monitoring
      //
      // Stock Monitoring
      //

      private void OnStockQuoteChange(object sender, StockUpdateEventArgs e)
      {
         if ( e.Symbol == _subscription.Symbol )
         {
            StockQuoteUpdate update = new StockQuoteUpdate();
            update.Symbol = e.Symbol;
            update.Quote = e.Quote;
            update.LastUpdate = e.UpdateTime;
            update.ClientId = _subscription.ClientId;

            Console.WriteLine("Notiying {0} about {1}", 
               update.ClientId, update.Symbol);

            _client.OnQuoteChanged(update);
         }
      }

      #endregion // Stock Monitoring

   } // class StockQuoteUpdateService

} // namespace Devdeo.StockQuotes.Service
