
//
// Program.cs
//
// Author:
//    Tomas Restrepo (tomas.restrepo@devdeo.com)
//
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Workflow.Activities;
using System.Workflow.Runtime;
using System.Workflow.Runtime.Hosting;

namespace Devdeo.StockQuotes.StockMonitor
{
   class Program
   {
      static void Main(string[] args)
      {
         WorkflowRuntime workflowRuntime = new WorkflowRuntime("Workflow");
         using ( workflowRuntime )
         {
            ExternalDataExchangeService svc = new ExternalDataExchangeService();
            workflowRuntime.AddService(svc);
            svc.AddService(new StockWatchService());

            AutoResetEvent waitHandle = new AutoResetEvent(false);
            workflowRuntime.WorkflowCompleted += 
               delegate(object sender, WorkflowCompletedEventArgs e) 
            { 
               waitHandle.Set(); 
            };
            workflowRuntime.WorkflowTerminated += 
               delegate(object sender, WorkflowTerminatedEventArgs e)
            {
               Console.WriteLine(e.Exception.Message);
               waitHandle.Set();
            };

            Dictionary<string, object> wfParams = GetParameters();
            WorkflowInstance instance = workflowRuntime.CreateWorkflow(
               typeof(StockMonitorWorkflow), wfParams
               );
            instance.Start();

            waitHandle.WaitOne();
         }
      }

      static Dictionary<string, object> GetParameters()
      {
         Dictionary<string, object> theParams = 
            new Dictionary<string, object>();
         theParams.Add("Symbol", "MSFT");
         theParams.Add("BuyThreshold", 27.00M);
         theParams.Add("SellThreshold", 35.00M);
         theParams.Add("Quantity", 10);

         return theParams;
      }

   } // class Program

} // namespace Devdeo.StockQuotes.StockMonitor

