
//
// StockMonitorWorkflow.cs
//
// Author:
//    Tomas Restrepo (tomas.restrepo@devdeo.com)
//
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
 
using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Drawing;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.Runtime;
using System.Workflow.Activities;
using System.Workflow.Activities.Rules;


namespace Devdeo.StockQuotes.StockMonitor
{

   /// <summary>
   /// State-machine workflow that monitors a Stock Symbol
   /// for changes in value, and acts accordingly
   /// </summary>
	public sealed partial class StockMonitorWorkflow 
      : StateMachineWorkflowActivity
	{
      private string _symbol;
      private decimal _buyThreshold;
      private decimal _sellThreshold;
      private int _quantity;

      public StockQuoteChangeEventArgs QuoteChange;
      public Guid WorkflowInstance;
      public StockOperation Operation;
      public const string QUEUE = @".\Private$\StockOperations";


      #region Properties
      //
      // Properties
      //

      public string Symbol
      {
         get { return _symbol; } 
         set { _symbol = value; } 
      }

      public decimal BuyThreshold
      {
         get { return _buyThreshold; }
         set { _buyThreshold = value; }
      }

      public decimal SellThreshold
      {
         get { return _sellThreshold; }
         set { _sellThreshold = value; }
      }

      public int Quantity
      {
         get { return _quantity; }
         set { _quantity = value; }
      }
      #endregion // Properties
      

      public StockMonitorWorkflow()
		{
         InitializeComponent();
		}

      private void PrepareSubscription(object sender, EventArgs e)
      {
         WorkflowInstance = WorkflowInstanceId;
      }

      private void PrintQuoteUpdate(object sender, EventArgs e)
      {
         Console.WriteLine("Received {0} quote: {1}",
            Symbol, QuoteChange.Quote);
      }

      private void PrepareBuyOperation(object sender, EventArgs e)
      {
         Console.WriteLine("Buying {0} {1} stocks at {2}", 
            Quantity, Symbol, QuoteChange.Quote);

         Operation = new StockOperation();
         Operation.Operation = StockOperation.BUY;
         Operation.Quantity = Quantity;
         Operation.Symbol = Symbol;
         Operation.Price = QuoteChange.Quote;
      }

      private void PrepareSellOperation(object sender, EventArgs e)
      {
         Console.WriteLine("Selling {0} {1} stocks at {2}", 
            Quantity, Symbol, QuoteChange.Quote);

         Operation = new StockOperation();
         Operation.Operation = StockOperation.SELL;
         Operation.Quantity = Quantity;
         Operation.Symbol = Symbol;
         Operation.Price = QuoteChange.Quote;
      }

	} // class StockMonitorWorkflow

} // namespace Devdeo.StockQuotes.StockMonitor
